( 3iOLED.fs -- 2020-10-08---Thu---21:02 )

0 constant BLACK    
1 constant WHITE    
2 constant INVERSE

$7F constant XMAX  \ 127
$3F constant YMAX  \ 63

$F  constant CALL_DISPLAYCLEAR
$10 constant CALL_TEXTSIZE
$11 constant CALL_TEXTCOLOR
$12 constant CALL_CURSOR
$13 constant CALL_PRINT
$14 constant CALL_DISPLAYSHOW
$15 constant CALL_PIXEL
$16 constant CALL_CHAR


: oled-update ( -- )     CALL_DISPLAYSHOW xcall ;

: oled-clear  ( -- )     CALL_DISPLAYCLEAR xcall ;
: oled-color  ( col -- ) xstore CALL_TEXTCOLOR xcall ;
: oled-textsz ( sz -- )  xstore CALL_TEXTSIZE xcall ;
: oled-cursor ( y x )    swap xstore xstore CALL_CURSOR xcall ;
: oled-print  ( -- )     CALL_PRINT xcall ;
: oled-pixel  ( y x col ) -rot swap xstore xstore xstore CALL_PIXEL xcall ;
: oled-char   ( ch -- ) xstore CALL_CHAR xcall ;
: oled-string ( c-adr n -- ) 0 do c@+ oled-char loop drop ;

: oled-init ( -- ) white oled-color 1 oled-textsz ;
: oled-page ( -- )  oled-clear 0 0 oled-cursor oled-update ;  \ new page set to defaults
: oled-dot  ( y x -- ) white oled-pixel 60 ms oled-update ;
: hline ( y -- ) XMAX 1 + 0 do dup i oled-dot loop drop ;
: vline ( x -- ) YMAX 1 + 0 do i over oled-dot loop drop ;
: quarters ( -- ) YMAX 2 / hline XMAX 2 / vline ;

: hello  'h' oled-char 'e' oled-char 'l' oled-char 'l' oled-char 'o' oled-char oled-update ;
: love  s" I love you very much!" oled-string oled-update ;
: big-love oled-page 5 oled-textsz s" love" oled-string oled-update ;
: big-adam oled-page 5 oled-textsz s" Adam" oled-string oled-update ;
: big-word ( cadr n -- ) 5 oled-textsz oled-page oled-string oled-update ;
: med-word ( cadr n -- ) 3 oled-textsz oled-page oled-string oled-update ;

\ EXERCISE: Text-i-talkie --- allow sending text messages using wireless version of Arduino Nano :)
: sz ( sz -- ) oled-page oled-textsz ;
: xword ( cadr n -- ) oled-string oled-update ;
: xcr ( -- ) $A oled-char ;
: xclear ( -- ) oled-page ;

\ EXERCISE: do the maths for all kinds of shapes (lines, circles, ellipses)
\ Fixed point arithmetic

\ EXERCISE: etch-a-sketch drawing controls, and interpolation between points...
\ connect D4 to LED with 220R resistor, then on to pin A7 (21) through 1.5KR resistor and to ground through 10K potentiometer
\ tactile button (pullup) at pin 5

\ 10 bit references
: etch-init  2 out 20 ms 3 out 20 ms 20 in 20 ms 21 in 20 ms 2 hi 3 hi 20 ms 5 pullup 20 ms ;
: Xref_calc  1024 xmax / 1 - ;
Xref_calc constant Xref
: Yref_calc  1024 ymax / 1 - ;
Yref_calc constant Yref
: readXlevel  21 pinreadLevel Xref / ; 
: readYlevel  20 pinreadLevel Yref / ; 
: xclear? ( --) 5 pinread 0= if oled-page 20 ms then ;
: etch oled-page begin readYlevel readXlevel oled-dot xclear? key? until ;

\ oscilloscope - scrolling in Y direction and plotting the level, refreshing the page when screen filled (later do scrolling)
: scroll? ( y -- y ) dup 0= if oled-page 20 ms then ;
: oled-level 0 begin scroll? dup readXlevel oled-dot 1 + ymax mod key?  until drop ;

\ EXERCISE: joystick is just 2 10k pots 
\ EXERCISE: add functionality within .AVR to read more efficiently for better responsiveness.
\ EXERCISE: reduce the nonlinearity of the pot action...

 
: oled-test  
    oled-clear oled-update 500 ms
    oled-print oled-update 500 ms
    oled-print oled-update 500 ms
    10 0  oled-cursor oled-print oled-update 500 ms
    20 5  oled-cursor 2 oled-textsz oled-print oled-update 500 ms
    30 10 oled-cursor 1 oled-textsz oled-print oled-update 500 ms
    40 20 oled-cursor 3 oled-textsz oled-print oled-update 500 ms
    1 oled-textsz
;


\ Version History - Started: 2020-10-08---Thu---21:02
\ v1 - 2020-10-09---Fri
\        FEATURE: Completed OLED display enablement, positioning, and pixel drawing
\ v2 - 2020-10-10---Sat---15:21
\       FEATURE: Completed character and string writing
\       FEATURE: Completed 2-dimensional OLED etch-a-sketch with two potentiometers!
\       FEATURE: Completed scrolling oscilloscope with 1-dimensional OLED reading pot
